# from https://medium.com/analytics-vidhya/optimized-ways-to-read-large-csvs-in-python-ab2b36a7914e
#from dask import dataframe as dd

# import Visualisation
# import FormattedExport
# install like this (according to https://docs.dask.org/en/latest/install.html#pip):
# pip install "dask[complete]"

# use dask to circumvent memory-issues, which occur according to https://community.brain-map.org/t/reading-rna-seq-data-into-python/658
#def read(path):
  # https://stackoverflow.com/questions/61647974/valueerror-sample-is-not-large-enough-to-include-at-least-one-row-of-data-plea
#  return dd.read_csv(urlpath=path, sample=256000 * 100)
  
# https://docs.dask.org/en/latest/dataframe.html
# https://brainmapportal-live-4cc80a57cd6e400d854-f7fdcae.divio-media.net/filer_public/70/32/70326830-e306-4743-a02c-a8da5bf9eb56/readme-m1-10.txt
# https://portal.brain-map.org/atlases-and-data/rnaseq
# https://portal.brain-map.org/atlases-and-data/rnaseq/human-m1-10x
# https://portal.brain-map.org/atlases-and-data/rnaseq/mouse-whole-cortex-and-hippocampus-10x
#rnaseq = read('data-rnaseq/aibs_human_m1_10x/matrix.csv')

#rnaseq.partitions[0].to_csv('export/rna-seq-0.xlsx')
#head = rnaseq.head(2).compute()
#FormattedExport.to_excel(rnaseq.partitions[0], 'export/rna-seq-0.xlsx')
#Visualisation.grid(head)

#print(rnaseq.columns)




from . import Visualisation
from . import Constants

import webbrowser
from threading import Timer

def setAvailableGenes(str):
  Constants.Genes = str

def start(port = 5000):

  def open_browser():
  	webbrowser.open_new("http://localhost:{}".format(port))

  webApp = Visualisation.WebInterface(__name__, port) 

  Timer(1, open_browser).start();
  webApp.run_server(debug=False)







from allensdk.core.reference_space_cache import ReferenceSpaceCache
import pandas as pd

class StructureMap:
    def __init__(self, reference_space_key = 'annotation/ccf_2017', resolution = 25):
        self.reference_space_key = reference_space_key
        self.resolution = resolution

    
    def get(self, structure_graph_id):
        # from: https://allensdk.readthedocs.io/en/latest/_static/examples/nb/reference_space.html#Constructing-a-structure-tree
        # http://help.brain-map.org/display/api/Atlas+Drawings+and+Ontologies

        # ! we need to use different FOLDERS for each structure-graph. else, structures.json gets overwritten by the last retrieved species/graph-id. yikes...
        rspc = ReferenceSpaceCache(self.resolution, self.reference_space_key, manifest=f'cache\\reference_space\\{structure_graph_id}\\cache.json')
        
        # doc @ http://api.brain-map.org/doc/Structure.html
        tree = rspc.get_structure_tree(structure_graph_id=structure_graph_id) 
        #annotation, meta = rspc.get_annotation_volume()

        # https://allensdk.readthedocs.io/en/latest/_static/examples/nb/reference_space.html
        name_map = tree.get_name_map()
        # from the readthedocs.io-sample: create a mapping from id to acronym.
        acronym_map = tree.value_map(lambda x: x['id'], lambda y: y['acronym']) # tree.get_id_acronym_map() # tree.value_map(lambda x: x['id'], lambda y: y['acronym'])
        
        # from https://datatofish.com/dictionary-to-dataframe/
        structure_map = pd.DataFrame(list(name_map.items()), columns = ['structure_id','structure_name']).set_index(['structure_id'])

        # the index (= structure-id) is mapped to an acronym:
        structure_map['acronym'] = structure_map.index.map(acronym_map)

        # this transforms the ancestor_id_map into a dataframe containing the structure_id and the respective ancestors.
        ancestor_map = pd.DataFrame([{
            "structure_id": k, 
            # ancestors of a structure are obtained as a list, using get_ancestor_id_map()
            # but instead of the ancestor's id, we want the name. hence, we lookup using the name_map
            "ancestors": [name_map[id] for id in reversed(v)] # we reverse the ancestors to always have the root in first position. 
            # from there, we go into finer structures from left to right. this way, we can have null-values for the root-nodes
            } for k, v in tree.get_ancestor_id_map().items()])

        ancestor_map = ancestor_map.set_index(['structure_id'])

        # ancestors are stored as a list in one column. but we want separate columns for each ancestor-level:
        # https://stackoverflow.com/questions/35491274/pandas-split-column-of-lists-into-multiple-columns
        ancestor_map = ancestor_map.ancestors.apply(pd.Series).add_prefix('level_')
        ancestor_map = ancestor_map.where(pd.notnull(ancestor_map), None)

        return structure_map.merge(ancestor_map, left_index=True, right_index=True, how="left").sort_index()


from setuptools import setup

setup()

import subprocess
import sys


# from https://stackoverflow.com/questions/12332975/installing-python-module-within-code
#subprocess.check_call([sys.executable, "-m", "pip", "install", "pandas==1.1.5"])




